/*====================================================================================
   This file is part of the TechTools Plugin Development Kit. 
   Copyright (c) 2011 by TechTools  

   You may modify this file and/or create derived works to develop plugins 
   for use with TechTools's DigiView software and capture files obtained from
   TechTools' hardware. Any other use is prohibited.

            email: support@tech-tools.com,  sales@tech-tools.com 
		web: www.tech-tools.com   Voice:972-272-9392  FAX: 972-494-5814
======================================================================================*/

// SimpleState.cpp 
// This is a simple example of a plugin using the built-in STATE parser as a
// preparser.  It demonstrates adding some options and different field formatting.
// This version ignores the field & frame SYNC bits and teh select events.  The pre-parser,
// however, honors the select settings (enable).

#include "../plugin.h"

// STATE pre-parser specific event flags
#define DATAEVENTFLAG   0x80
#define SELECTEVENTFLAG 0x40
#define SYNCEVENTFLAG   0x20
#define SYNC2EVENTFLAG  0x10

// A little short-hand... These give readable names to the Configuration Options and
// field formats described below.  The final entries are used as sentinals to mark the length/ends  

//  configuration items (case 1)
static enum cfgnames { SKIP,STATES_PER_FRAME,FIELD_IDLE,FRAME_IDLE,CFGEND };
// field formats (case 2)
static enum fieldformats { F_FIRST, F_OTHER, F_LAST, FORMATEND };

// unused stubs
void OnLoad() {}
void SetInitItem(unsigned char ID, unsigned char subID, int value) {}
void EndOfData() {}
void OnUnLoad() {}


// called to retrieve descriptions and configuration information from the plug-in
// fills the provided vector container with the requested set of strings
void GetStrList(int ID, vector<string> &strl)
{
	// You are free to store your strings any way you wish as long as you
	// copy them into the provided vector when requested. You might choose
	// to store them as constant arrays or pre-initialized vectors.  We chose
	//to load them directly in-line here, keeping all configuration in one place.
	switch (ID)
	{ 
	case 0:   // Plug-in Description
		// These lines are displayed in the signal editor for identification
		// but serve no other purpose. For best results, keep to 4 or fewer lines.
		strl.push_back(	"Simple State Plugin Example V0.1 BETA");
		break;

	case 1:   // configuration options
		  // These are the items displayed in the SIGNAL EDITOR. They allow the
  		  // user to configure this plug-in for a given signal.
		strl.assign(CFGEND,"");  // allocate and init the list
		strl[SKIP]             = "Skip States,spinner,0,0,255,1";
		strl[STATES_PER_FRAME] = "States per Frame,edit,0";
        strl[FIELD_IDLE]       = "Show Field Idle,checkbox,false";
		strl[FRAME_IDLE]       = "Show Frame Idle,checkbox,false";
		break;

	case 2:  // Field Formats
		  // These tell the app how to display your fields. You must define at least 
		  // one field format.  format= name,fieldcolor,textcolor,display text/data
		strl.assign(FORMATEND,"");  // allocate and init the list
		strl[F_FIRST] =  "First Field,BLUE,WHITE,{}";
		strl[F_OTHER] =  "Other,c0c0c0,BLACK,{}";
		strl[F_LAST] =   "Last,YELLOW,BLACK,{}";
		break;

	case 3:  //return which preparser we need
		strl.push_back("STATE");
		break;

	case 4: // return minimim framework ver we need
		strl.push_back("1");
		break;
	}
}


static bool first_sample;
static int  states_per_frame;
static int  skip_count;
static bool ShowFieldIdle;
static bool ShowFrameIdle;
static int  FieldNum;        // tracks states per frame

// called to set our configuration options.
// These represent the choices the user made in response to your Configuration Options
// You can store and retrieve your configuration items any way you want.
// We chose to store them directly in named variables.
// The ID corresponds to the index of the Configuration Options item.
// eg 0 => the first configuration string you provided....
// We used enums here and in the cfg option definitions for consistency and readability.

void SetCfgItem(unsigned char ID, unsigned char subID, int value)
{
	switch (ID)
	{
	case SKIP:				if (subID == 0) skip_count = int(value); break;
	case STATES_PER_FRAME:	if (subID == 0) states_per_frame = int(value);	break;
	case FIELD_IDLE:		if (subID == 0) ShowFieldIdle = (value != 0);	break;
	case FRAME_IDLE:		if (subID == 0) ShowFrameIdle = (value != 0); break;
	// ignore illegal cfg items
	}
}


// Called immediately after setting configuration options and before data
// streaming begins. Do any initialization you require here. In particular,
// note that you should assume that the configuration options have changed.
void StartOfData()
{	FieldNum = skip_count;
}


// This is the main parser. It accepts a steam of events from the preparser and generates
// a stream of field specifiers (format and data via WriteField() ).
// It could get called 250K or more times for each signal using this plugin so efficiency is important.
void Parse(int64 timestamp, Data64 mydata)
{	
	// This mini-parser only cares about DATA EVENTs. It will ignore any other potential events
    if(mydata.bytes[6] & DATAEVENTFLAG)
	{
		// If the user specified 0 states per frame, it implies NO FRAMING
		if (states_per_frame < 1)  // if not using frames
		{
			StartField(timestamp, mydata, F_OTHER);  // use OTHER format definition, no frame
			if (ShowFieldIdle)  EndField(timestamp);
		}
		else
		{   FieldNum += 1;
			if (FieldNum == 1)   // start a new frame&field and use F_FIRST for formatting
			  StartFrame(timestamp, mydata, F_FIRST);
			else  if (FieldNum >= states_per_frame)  
			  StartField(timestamp, mydata, F_LAST); 
			else
			  StartField(timestamp, mydata, F_OTHER); 

			if (FieldNum >= states_per_frame)
			{	
				FieldNum = 0; 
				if (ShowFrameIdle)  EndFrame(timestamp);
			}
			else if (ShowFieldIdle) EndField(timestamp);
		} // else using frames
	}
}

